import logging
from urllib.parse import urlparse
import requests
import redis
import os
from dotenv import load_dotenv
from typing import Optional, Dict, Any
import json

# Load environment variables from .env file
load_dotenv()

# Configure logging
logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)

# Initialize Redis client
redis_client = None
if os.getenv('REDIS_HOST') and os.getenv('REDIS_PORT') and os.getenv('REDIS_DB'):
    redis_client = redis.StrictRedis(
        host=os.getenv('REDIS_HOST', 'localhost'),
        port=int(os.getenv('REDIS_PORT', 6379)),
        db=int(os.getenv('REDIS_DB', 0)),
        password=os.getenv('REDIS_PASSWORD', None)
    )

class GeoLocation:
    def __init__(self, proxy: Optional[str] = None):
        self.proxy = proxy
        self.ip_information = None

    def get_correct_region_code(self, timezone: str) -> Optional[str]:
        timezone_to_region = {
            'Europe/Andorra': 'AD',
            'Asia/Dubai': 'AE',
            'Asia/Kabul': 'AF',
            'Europe/Tirane': 'AL',
            'Asia/Yerevan': 'AM',
            'Antarctica/Casey': 'AQ',
            'Antarctica/Davis': 'AQ',
            'Antarctica/DumontDUrville': 'AQ',
            'Antarctica/Mawson': 'AQ',
            'Antarctica/Palmer': 'AQ',
            'Antarctica/Rothera': 'AQ',
            'Antarctica/Syowa': 'AQ',
            'Antarctica/Troll': 'AQ',
            'Antarctica/Vostok': 'AQ',
            'America/Argentina/Buenos_Aires': 'AR',
            'America/Argentina/Cordoba': 'AR',
            'America/Argentina/Salta': 'AR',
            'America/Argentina/Jujuy': 'AR',
            'America/Argentina/Tucuman': 'AR',
            'America/Argentina/Catamarca': 'AR',
            'America/Argentina/La_Rioja': 'AR',
            'America/Argentina/San_Juan': 'AR',
            'America/Argentina/Mendoza': 'AR',
            'America/Argentina/San_Luis': 'AR',
            'America/Argentina/Rio_Gallegos': 'AR',
            'America/Argentina/Ushuaia': 'AR',
            'Pacific/Pago_Pago': 'AS',
            'Europe/Vienna': 'AT',
            'Australia/Lord_Howe': 'AU',
            'Antarctica/Macquarie': 'AU',
            'Australia/Hobart': 'AU',
            'Australia/Currie': 'AU',
            'Australia/Melbourne': 'AU',
            'Australia/Sydney': 'AU',
            'Australia/Broken_Hill': 'AU',
            'Australia/Brisbane': 'AU',
            'Australia/Lindeman': 'AU',
            'Australia/Adelaide': 'AU',
            'Australia/Darwin': 'AU',
            'Australia/Perth': 'AU',
            'Australia/Eucla': 'AU',
            'Asia/Baku': 'AZ',
            'America/Barbados': 'BB',
            'Asia/Dhaka': 'BD',
            'Europe/Brussels': 'BE',
            'Europe/Sofia': 'BG',
            'Atlantic/Bermuda': 'BM',
            'Asia/Brunei': 'BN',
            'America/La_Paz': 'BO',
            'America/Noronha': 'BR',
            'America/Belem': 'BR',
            'America/Fortaleza': 'BR',
            'America/Recife': 'BR',
            'America/Araguaina': 'BR',
            'America/Maceio': 'BR',
            'America/Bahia': 'BR',
            'America/Sao_Paulo': 'BR',
            'America/Campo_Grande': 'BR',
            'America/Cuiaba': 'BR',
            'America/Santarem': 'BR',
            'America/Porto_Velho': 'BR',
            'America/Boa_Vista': 'BR',
            'America/Manaus': 'BR',
            'America/Eirunepe': 'BR',
            'America/Rio_Branco': 'BR',
            'America/Nassau': 'BS',
            'Asia/Thimphu': 'BT',
            'Europe/Minsk': 'BY',
            'America/Belize': 'BZ',
            'America/St_Johns': 'CA',
            'America/Halifax': 'CA',
            'America/Glace_Bay': 'CA',
            'America/Moncton': 'CA',
            'America/Goose_Bay': 'CA',
            'America/Blanc-Sablon': 'CA',
            'America/Toronto': 'CA',
            'America/Nipigon': 'CA',
            'America/Thunder_Bay': 'CA',
            'America/Iqaluit': 'CA',
            'America/Pangnirtung': 'CA',
            'America/Atikokan': 'CA',
            'America/Winnipeg': 'CA',
            'America/Rainy_River': 'CA',
            'America/Resolute': 'CA',
            'America/Rankin_Inlet': 'CA',
            'America/Regina': 'CA',
            'America/Swift_Current': 'CA',
            'America/Edmonton': 'CA',
            'America/Cambridge_Bay': 'CA',
            'America/Yellowknife': 'CA',
            'America/Inuvik': 'CA',
            'America/Creston': 'CA',
            'America/Dawson_Creek': 'CA',
            'America/Fort_Nelson': 'CA',
            'America/Vancouver': 'CA',
            'America/Whitehorse': 'CA',
            'America/Dawson': 'CA',
            'Indian/Cocos': 'CC',
            'Europe/Zurich': 'CH',
            'Africa/Abidjan': 'CI',
            'Pacific/Rarotonga': 'CK',
            'America/Santiago': 'CL',
            'America/Punta_Arenas': 'CL',
            'Pacific/Easter': 'CL',
            'Asia/Shanghai': 'CN',
            'Asia/Urumqi': 'CN',
            'America/Bogota': 'CO',
            'America/Costa_Rica': 'CR',
            'America/Havana': 'CU',
            'Atlantic/Cape_Verde': 'CV',
            'America/Curacao': 'CW',
            'Indian/Christmas': 'CX',
            'Asia/Nicosia': 'CY',
            'Asia/Famagusta': 'CY',
            'Europe/Prague': 'CZ',
            'Europe/Berlin': 'DE',
            'Europe/Copenhagen': 'DK',
            'America/Santo_Domingo': 'DO',
            'Africa/Algiers': 'DZ',
            'America/Guayaquil': 'EC',
            'Pacific/Galapagos': 'EC',
            'Europe/Tallinn': 'EE',
            'Africa/Cairo': 'EG',
            'Africa/El_Aaiun': 'EH',
            'Europe/Madrid': 'ES',
            'Africa/Ceuta': 'ES',
            'Atlantic/Canary': 'ES',
            'Europe/Helsinki': 'FI',
            'Pacific/Fiji': 'FJ',
            'Atlantic/Stanley': 'FK',
            'Pacific/Chuuk': 'FM',
            'Pacific/Pohnpei': 'FM',
            'Pacific/Kosrae': 'FM',
            'Atlantic/Faroe': 'FO',
            'Europe/Paris': 'FR',
            'Europe/London': 'GB',
            'Asia/Tbilisi': 'GE',
            'America/Cayenne': 'GF',
            'Africa/Accra': 'GH',
            'Europe/Gibraltar': 'GI',
            'America/Godthab': 'GL',
            'America/Danmarkshavn': 'GL',
            'America/Scoresbysund': 'GL',
            'America/Thule': 'GL',
            'Europe/Athens': 'GR',
            'Atlantic/South_Georgia': 'GS',
            'America/Guatemala': 'GT',
            'Pacific/Guam': 'GU',
            'Africa/Bissau': 'GW',
            'America/Guyana': 'GY',
            'Asia/Hong_Kong': 'HK',
            'America/Tegucigalpa': 'HN',
            'America/Port-au-Prince': 'HT',
            'Europe/Budapest': 'HU',
            'Asia/Jakarta': 'ID',
            'Asia/Pontianak': 'ID',
            'Asia/Makassar': 'ID',
            'Asia/Jayapura': 'ID',
            'Europe/Dublin': 'IE',
            'Asia/Jerusalem': 'IL',
            'Asia/Kolkata': 'IN',
            'Indian/Chagos': 'IO',
            'Asia/Baghdad': 'IQ',
            'Asia/Tehran': 'IR',
            'Atlantic/Reykjavik': 'IS',
            'Europe/Rome': 'IT',
            'Asia/Jamaica': 'JM',
            'Asia/Amman': 'JO',
            'Asia/Tokyo': 'JP',
            'Africa/Nairobi': 'KE',
            'Asia/Bishkek': 'KG',
            'Pacific/Tarawa': 'KI',
            'Pacific/Enderbury': 'KI',
            'Pacific/Kiritimati': 'KI',
            'Asia/Pyongyang': 'KP',
            'Asia/Seoul': 'KR',
            'Asia/Almaty': 'KZ',
            'Asia/Qyzylorda': 'KZ',
            'Asia/Qostanay': 'KZ',
            'Asia/Aqtobe': 'KZ',
            'Asia/Aqtau': 'KZ',
            'Asia/Atyrau': 'KZ',
            'Asia/Oral': 'KZ',
            'Asia/Beirut': 'LB',
            'Asia/Colombo': 'LK',
            'Africa/Monrovia': 'LR',
            'Europe/Vilnius': 'LT',
            'Europe/Luxembourg': 'LU',
            'Europe/Riga': 'LV',
            'Africa/Tripoli': 'LY',
            'Africa/Casablanca': 'MA',
            'Europe/Monaco': 'MC',
            'Europe/Chisinau': 'MD',
            'Pacific/Majuro': 'MH',
            'Pacific/Kwajalein': 'MH',
            'Asia/Yangon': 'MM',
            'Asia/Ulaanbaatar': 'MN',
            'Asia/Hovd': 'MN',
            'Asia/Choibalsan': 'MN',
            'Asia/Macau': 'MO',
            'America/Martinique': 'MQ',
            'Europe/Malta': 'MT',
            'Indian/Mauritius': 'MU',
            'Indian/Maldives': 'MV',
            'America/Mexico_City': 'MX',
            'America/Cancun': 'MX',
            'America/Merida': 'MX',
            'America/Monterrey': 'MX',
            'America/Matamoros': 'MX',
            'America/Mazatlan': 'MX',
            'America/Chihuahua': 'MX',
            'America/Ojinaga': 'MX',
            'America/Hermosillo': 'MX',
            'America/Tijuana': 'MX',
            'America/Bahia_Banderas': 'MX',
            'Asia/Kuala_Lumpur': 'MY',
            'Asia/Kuching': 'MY',
            'Africa/Maputo': 'MZ',
            'Africa/Windhoek': 'NA',
            'Pacific/Noumea': 'NC',
            'Pacific/Norfolk': 'NF',
            'Africa/Lagos': 'NG',
            'America/Managua': 'NI',
            'Europe/Amsterdam': 'NL',
            'Europe/Oslo': 'NO',
            'Asia/Kathmandu': 'NP',
            'Pacific/Nauru': 'NR',
            'Pacific/Niue': 'NU',
            'Pacific/Auckland': 'NZ',
            'Pacific/Chatham': 'NZ',
            'America/Panama': 'PA',
            'America/Lima': 'PE',
            'Pacific/Tahiti': 'PF',
            'Pacific/Marquesas': 'PF',
            'Pacific/Gambier': 'PF',
            'Pacific/Port_Moresby': 'PG',
            'Pacific/Bougainville': 'PG',
            'Asia/Manila': 'PH',
            'Asia/Karachi': 'PK',
            'Europe/Warsaw': 'PL',
            'America/Miquelon': 'PM',
            'Pacific/Pitcairn': 'PN',
            'America/Puerto_Rico': 'PR',
            'Asia/Gaza': 'PS',
            'Asia/Hebron': 'PS',
            'Europe/Lisbon': 'PT',
            'Atlantic/Madeira': 'PT',
            'Atlantic/Azores': 'PT',
            'Pacific/Palau': 'PW',
            'America/Asuncion': 'PY',
            'Asia/Qatar': 'QA',
            'Indian/Reunion': 'RE',
            'Europe/Bucharest': 'RO',
            'Europe/Belgrade': 'RS',
            'Europe/Kaliningrad': 'RU',
            'Europe/Moscow': 'RU',
            'Europe/Simferopol': 'RU',
            'Europe/Kirov': 'RU',
            'Europe/Astrakhan': 'RU',
            'Europe/Volgograd': 'RU',
            'Europe/Saratov': 'RU',
            'Europe/Ulyanovsk': 'RU',
            'Europe/Samara': 'RU',
            'Asia/Yekaterinburg': 'RU',
            'Asia/Omsk': 'RU',
            'Asia/Novosibirsk': 'RU',
            'Asia/Barnaul': 'RU',
            'Asia/Tomsk': 'RU',
            'Asia/Novokuznetsk': 'RU',
            'Asia/Krasnoyarsk': 'RU',
            'Asia/Irkutsk': 'RU',
            'Asia/Chita': 'RU',
            'Asia/Yakutsk': 'RU',
            'Asia/Khandyga': 'RU',
            'Asia/Vladivostok': 'RU',
            'Asia/Ust-Nera': 'RU',
            'Asia/Magadan': 'RU',
            'Asia/Sakhalin': 'RU',
            'Asia/Srednekolymsk': 'RU',
            'Asia/Kamchatka': 'RU',
            'Asia/Anadyr': 'RU',
            'Asia/Riyadh': 'SA',
            'Pacific/Guadalcanal': 'SB',
            'Indian/Mahe': 'SC',
            'Africa/Khartoum': 'SD',
            'Europe/Stockholm': 'SE',
            'Asia/Singapore': 'SG',
            'America/Paramaribo': 'SR',
            'Africa/Juba': 'SS',
            'Africa/Sao_Tome': 'ST',
            'America/El_Salvador': 'SV',
            'Asia/Damascus': 'SY',
            'America/Grand_Turk': 'TC',
            'Africa/Ndjamena': 'TD',
            'Indian/Kerguelen': 'TF',
            'Asia/Bangkok': 'TH',
            'Asia/Dushanbe': 'TJ',
            'Pacific/Fakaofo': 'TK',
            'Asia/Dili': 'TL',
            'Asia/Ashgabat': 'TM',
            'Africa/Tunis': 'TN',
            'Pacific/Tongatapu': 'TO',
            'Europe/Istanbul': 'TR',
            'America/Port_of_Spain': 'TT',
            'Pacific/Funafuti': 'TV',
            'Asia/Taipei': 'TW',
            'Europe/Kiev': 'UA',
            'Europe/Uzhgorod': 'UA',
            'Europe/Zaporozhye': 'UA',
            'Pacific/Wake': 'UM',
            'America/New_York': 'US',
            'America/Detroit': 'US',
            'America/Kentucky/Louisville': 'US',
            'America/Kentucky/Monticello': 'US',
            'America/Indiana/Indianapolis': 'US',
            'America/Indiana/Vincennes': 'US',
            'America/Indiana/Winamac': 'US',
            'America/Indiana/Marengo': 'US',
            'America/Indiana/Petersburg': 'US',
            'America/Indiana/Vevay': 'US',
            'America/Chicago': 'US',
            'America/Indiana/Tell_City': 'US',
            'America/Indiana/Knox': 'US',
            'America/Menominee': 'US',
            'America/North_Dakota/Center': 'US',
            'America/North_Dakota/New_Salem': 'US',
            'America/North_Dakota/Beulah': 'US',
            'America/Denver': 'US',
            'America/Boise': 'US',
            'America/Phoenix': 'US',
            'America/Los_Angeles': 'US',
            'America/Anchorage': 'US',
            'America/Juneau': 'US',
            'America/Sitka': 'US',
            'America/Metlakatla': 'US',
            'America/Yakutat': 'US',
            'America/Nome': 'US',
            'America/Adak': 'US',
            'Pacific/Honolulu': 'US',
            'America/Montevideo': 'UY',
            'Asia/Samarkand': 'UZ',
            'Asia/Tashkent': 'UZ',
            'America/Caracas': 'VE',
            'Asia/Ho_Chi_Minh': 'VN',
            'Pacific/Efate': 'VU',
            'Pacific/Wallis': 'WF',
            'Pacific/Apia': 'WS',
            'Africa/Johannesburg': 'ZA',
        }

        logger.debug(f"found timezone {timezone}")
        region_from_timezone = timezone_to_region.get(timezone)
        logger.debug(f"Could find region code for {region_from_timezone}")
        return region_from_timezone

    def get_ip(self) -> str:
        url = 'http://143.198.60.242'
        proxies = None
        if self.proxy:
            proxy_parts = urlparse(self.proxy)
            proxies = {
                "http": f"http://{proxy_parts.username}:{proxy_parts.password}@{proxy_parts.hostname}:{proxy_parts.port}",
                "https": f"http://{proxy_parts.username}:{proxy_parts.password}@{proxy_parts.hostname}:{proxy_parts.port}"
            }
        response = requests.get(url, proxies=proxies)
        response.raise_for_status()
        return response.text.strip()
    
    def get_ip_data(self, ip: str) -> Dict[str, Any]:
        cache_key = f"ipdata:{ip}"
        if redis_client:
            cached_data = redis_client.get(cache_key)
            if cached_data:
                logger.debug(f"Found cached IP data for {ip}")
                return json.loads(cached_data)

        url = f"http://ip-api.com/json/{ip}?fields=17035263"
        proxies = None
        if self.proxy:
            proxy_parts = urlparse(self.proxy)
            proxies = {
                "http": f"http://{proxy_parts.username}:{proxy_parts.password}@{proxy_parts.hostname}:{proxy_parts.port}",
                "https": f"http://{proxy_parts.username}:{proxy_parts.password}@{proxy_parts.hostname}:{proxy_parts.port}"
            }
        response = requests.get(url, proxies=proxies)
        response.raise_for_status()
        data = response.json()

        # Cache the result for 30 days if Redis is available
        if redis_client:
            redis_client.setex(cache_key, 60 * 60 * 24 * 30, json.dumps(data))
        return data

    def get_geolocation(self) -> Dict[str, Any]:
        ip = self.get_ip()
        logger.debug(f"Fetched IP: {ip}")
            
        self.ip_information = self.get_ip_data(ip)
        logger.debug(f"found ip data {self.ip_information}")

        lat = self.ip_information.get('lat')
        lon = self.ip_information.get('lon')
        tz = self.ip_information.get('timezone')
        locale = self.ip_information.get('countryCode').lower()
        region = self.get_correct_region_code(tz)

        random_locale = 'en-US'  # Default fallback
        language_dict = {
            "AF": ["pr-AF", "pr"],
            "AX": ["sw-AX", "sw"],
            "AL": ["sq-AL", "sq"],
            "DZ": ["ar-DZ", "ar"],
            "AS": ["en-AS", "en"],
            "AD": ["ca-AD", "ca"],
            "AO": ["po-AO", "po"],
            "AI": ["en-AI", "en"],
            "AG": ["en-AG", "en"],
            "AR": ["gr-AR", "gr"],
            "AM": ["hy-AM", "hy"],
            "AW": ["nl-AW", "nl"],
            "AU": ["en-AU", "en"],
            "AT": ["ba-AT", "ba"],
            "AZ": ["az-AZ", "az"],
            "BS": ["en-BS", "en"],
            "BH": ["ar-BH", "ar"],
            "BD": ["be-BD", "be"],
            "BB": ["en-BB", "en"],
            "BY": ["be-BY", "be"],
            "BE": ["de-BE", "de"],
            "BQ": ["en-BQ", "en"],
            "BZ": ["bj-BZ", "bj"],
            "BJ": ["fr-BJ", "fr"],
            "BM": ["en-BM", "en"],
            "BT": ["dz-BT", "dz"],
            "BO": ["ay-BO", "ay"],
            "BA": ["bo-BA", "bo"],
            "BW": ["en-BW", "en"],
            "BV": ["no-BV", "no"],
            "BR": ["po-BR", "po"],
            "IO": ["en-IO", "en"],
            "BN": ["ms-BN", "ms"],
            "BG": ["bu-BG", "bu"],
            "BF": ["fr-BF", "fr"],
            "BI": ["fr-BI", "fr"],
            "KH": ["kh-KH", "kh"],
            "CM": ["en-CM", "en"],
            "CA": ["en-CA", "en"],
            "CV": ["po-CV", "po"],
            "KY": ["en-KY", "en"],
            "CF": ["fr-CF", "fr"],
            "TD": ["ar-TD", "ar"],
            "CL": ["sp-CL", "sp"],
            "CN": ["zh-CN", "zh"],
            "CX": ["en-CX", "en"],
            "CC": ["en-CC", "en"],
            "CO": ["sp-CO", "sp"],
            "KM": ["ar-KM", "ar"],
            "CG": ["fr-CG", "fr"],
            "CD": ["fr-CD", "fr"],
            "CK": ["en-CK", "en"],
            "CR": ["sp-CR", "sp"],
            "CI": ["fr-CI", "fr"],
            "HR": ["hr-HR", "hr"],
            "CU": ["sp-CU", "sp"],
            "CW": ["en-CW", "en"],
            "CY": ["el-CY", "el"],
            "CZ": ["ce-CZ", "ce"],
            "DK": ["da-DK", "da"],
            "DJ": ["ar-DJ", "ar"],
            "DM": ["en-DM", "en"],
            "DO": ["sp-DO", "sp"],
            "EC": ["sp-EC", "sp"],
            "EG": ["ar-EG", "ar"],
            "SV": ["sp-SV", "sp"],
            "GQ": ["fr-GQ", "fr"],
            "ER": ["ar-ER", "ar"],
            "EE": ["es-EE", "es"],
            "ET": ["am-ET", "am"],
            "FK": ["en-FK", "en"],
            "FO": ["da-FO", "da"],
            "FJ": ["en-FJ", "en"],
            "FI": ["fi-FI", "fi"],
            "FR": ["fr-FR", "fr"],
            "GF": ["fr-GF", "fr"],
            "PF": ["fr-PF", "fr"],
            "TF": ["fr-TF", "fr"],
            "GA": ["fr-GA", "fr"],
            "GM": ["en-GM", "en"],
            "GE": ["ka-GE", "ka"],
            "DE": ["de-DE", "de"],
            "GH": ["en-GH", "en"],
            "GI": ["en-GI", "en"],
            "GR": ["el-GR", "el"],
            "GL": ["ka-GL", "ka"],
            "GD": ["en-GD", "en"],
            "GP": ["fr-GP", "fr"],
            "GU": ["ch-GU", "ch"],
            "GT": ["sp-GT", "sp"],
            "GG": ["en-GG", "en"],
            "GN": ["fr-GN", "fr"],
            "GW": ["po-GW", "po"],
            "GY": ["en-GY", "en"],
            "HT": ["fr-HT", "fr"],
            "HM": ["en-HM", "en"],
            "VA": ["it-VA", "it"],
            "HN": ["sp-HN", "sp"],
            "HK": ["en-HK", "en"],
            "HU": ["hu-HU", "hu"],
            "IS": ["is-IS", "is"],
            "IN": ["en-IN", "en"],
            "ID": ["in-ID", "in"],
            "IR": ["fa-IR", "fa"],
            "IQ": ["ar-IQ", "ar"],
            "IE": ["en-IE", "en"],
            "IM": ["en-IM", "en"],
            "IL": ["ar-IL", "ar"],
            "IT": ["it-IT", "it"],
            "JM": ["en-JM", "en"],
            "JP": ["jp-JP", "jp"],
            "JE": ["en-JE", "en"],
            "JO": ["ar-JO", "ar"],
            "KZ": ["ka-KZ", "ka"],
            "KE": ["en-KE", "en"],
            "KI": ["en-KI", "en"],
            "KP": ["ko-KP", "ko"],
            "KR": ["ko-KR", "ko"],
            "KW": ["ar-KW", "ar"],
            "KG": ["ki-KG", "ki"],
            "LA": ["la-LA", "la"],
            "LV": ["la-LV", "la"],
            "LB": ["ar-LB", "ar"],
            "LS": ["en-LS", "en"],
            "LR": ["en-LR", "en"],
            "LY": ["ar-LY", "ar"],
            "LI": ["de-LI", "de"],
            "LT": ["li-LT", "li"],
            "LU": ["de-LU", "de"],
            "MO": ["po-MO", "po"],
            "MK": ["mk-MK", "mk"],
            "MG": ["fr-MG", "fr"],
            "MW": ["en-MW", "en"],
            "MY": ["en-MY", "en"],
            "MV": ["di-MV", "di"],
            "ML": ["fr-ML", "fr"],
            "MT": ["en-MT", "en"],
            "MH": ["en-MH", "en"],
            "MQ": ["fr-MQ", "fr"],
            "MR": ["ar-MR", "ar"],
            "MU": ["en-MU", "en"],
            "YT": ["fr-YT", "fr"],
            "MX": ["sp-MX", "sp"],
            "FM": ["en-FM", "en"],
            "MD": ["ro-MD", "ro"],
            "MC": ["fr-MC", "fr"],
            "MN": ["mo-MN", "mo"],
            "MS": ["en-MS", "en"],
            "MA": ["ar-MA", "ar"],
            "MZ": ["po-MZ", "po"],
            "MM": ["my-MM", "my"],
            "NA": ["af-NA", "af"],
            "NR": ["en-NR", "en"],
            "NP": ["ne-NP", "ne"],
            "NL": ["nl-NL", "nl"],
            "NC": ["fr-NC", "fr"],
            "NZ": ["en-NZ", "en"],
            "NI": ["sp-NI", "sp"],
            "NE": ["fr-NE", "fr"],
            "NG": ["en-NG", "en"],
            "NU": ["en-NU", "en"],
            "NF": ["en-NF", "en"],
            "MP": ["ca-MP", "ca"],
            "NO": ["nn-NO", "nn"],
            "OM": ["ar-OM", "ar"],
            "PK": ["en-PK", "en"],
            "PW": ["en-PW", "en"],
            "PS": ["ar-PS", "ar"],
            "PA": ["sp-PA", "sp"],
            "PG": ["en-PG", "en"],
            "PY": ["gr-PY", "gr"],
            "PE": ["ay-PE", "ay"],
            "PH": ["en-PH", "en"],
            "PN": ["en-PN", "en"],
            "PL": ["po-PL", "po"],
            "PT": ["po-PT", "po"],
            "PR": ["en-PR", "en"],
            "QA": ["ar-QA", "ar"],
            "RE": ["fr-RE", "fr"],
            "RO": ["ro-RO", "ro"],
            "RU": ["ru-RU", "ru"],
            "RW": ["en-RW", "en"],
            "SH": ["en-SH", "en"],
            "KN": ["en-KN", "en"],
            "LC": ["en-LC", "en"],
            "PM": ["fr-PM", "fr"],
            "VC": ["en-VC", "en"],
            "WS": ["en-WS", "en"],
            "SM": ["it-SM", "it"],
            "ST": ["po-ST", "po"],
            "SA": ["ar-SA", "ar"],
            "SN": ["fr-SN", "fr"],
            "SC": ["cr-SC", "cr"],
            "SL": ["en-SL", "en"],
            "SG": ["zh-SG", "zh"],
            "SK": ["sl-SK", "sl"],
            "SI": ["sl-SI", "sl"],
            "SB": ["en-SB", "en"],
            "SO": ["ar-SO", "ar"],
            "SS": ["en-SS", "en"],
            "SX": ["en-SX", "en"],
            "ZA": ["af-ZA", "af"],
            "GS": ["en-GS", "en"],
            "ES": ["sp-ES", "sp"],
            "LK": ["si-LK", "si"],
            "SD": ["ar-SD", "ar"],
            "SR": ["nl-SR", "nl"],
            "SJ": ["no-SJ", "no"],
            "SZ": ["en-SZ", "en"],
            "SE": ["sw-SE", "sw"],
            "CH": ["fr-CH", "fr"],
            "SY": ["ar-SY", "ar"],
            "TW": ["zh-TW", "zh"],
            "TJ": ["ru-TJ", "ru"],
            "TZ": ["en-TZ", "en"],
            "TH": ["th-TH", "th"],
            "TL": ["po-TL", "po"],
            "TG": ["fr-TG", "fr"],
            "TK": ["en-TK", "en"],
            "TO": ["en-TO", "en"],
            "TT": ["en-TT", "en"],
            "TN": ["ar-TN", "ar"],
            "TR": ["tu-TR", "tu"],
            "TM": ["ru-TM", "ru"],
            "TC": ["en-TC", "en"],
            "TV": ["en-TV", "en"],
            "UG": ["en-UG", "en"],
            "UA": ["uk-UA", "uk"],
            "AE": ["ar-AE", "ar"],
            "GB": ["en-GB", "en"],
            "US": ["en-US", "en"],
            "UM": ["en-UM", "en"],
            "UY": ["sp-UY", "sp"],
            "UZ": ["ru-UZ", "ru"],
            "VU": ["bi-VU", "bi"],
            "VE": ["sp-VE", "sp"],
            "VN": ["vi-VN", "vi"],
            "VG": ["en-VG", "en"],
            "VI": ["en-VI", "en"],
            "WF": ["fr-WF", "fr"],
            "EH": ["be-EH", "be"],
            "YE": ["ar-YE", "ar"],
            "ZM": ["en-ZM", "en"],
            "ZW": ["bw-ZW", "bw"],
            "RS": ["sr-RS", "sr"],
            "ME": ["cn-ME", "cn"],
            "XK": ["sq-XK", "sq"],
        }

        if region and region in language_dict:
            random_locale = language_dict[region][0]

        logger.debug(f"found random locale {random_locale}")

        return {
            "lat": lat,
            "lon": lon,
            "timezone": tz,
            "locale": random_locale,
            "ip": ip
        }

# Example usage:
# geo = GeoLocation(proxy="http://user:pass@host:port")
# geolocation_data = geo.get_geolocation()
# print(geolocation_data)